;**********************************************************************************************************
;				I2C Slave Mode Header File For 16C71
;**********************************************************************************************************

#define	_CLOCK_STRETCH_CHECK	TRUE


;
;  All setup & hold times are @ 16 Mhz input clock.
;
#define	_t_HDSTA	16		; START hold time
#define	_t_SUSTA	19		; START setup time
#define	_t_HIGH		16	
#define	_t_LOW		19
#define	_t_SUSTO	16
#define	_t_BUF		20

#define	_40uS_Delay	13
#define	_47uS_Delay	16
#define	_50uS_Delay	17

;
#define	_WRITE_		0		        
#define	_READ_		1

;					Register File Variables


		CBLOCK	0x0C
                	SlvStatus		; Receive Status Byte
			DataByte		; load this reg with the data to be transmitted
                        ControlByte		; Slave's Command Reg
			BitCount		; The bit number (0:7) transmitted or received
			DelayCount
			DataByteCopy		; copy of DataByte for Left Shifts (destructive)

			SubAddr			; sub-address of slave (used in I2C_HIGH.ASM)
                        SrcPtr			; source pointer for data to be transmitted

			tempCount		; a temp variable for scratch RAM
			StoreTemp_1		; a temp variable for scratch RAM, do not disturb contents

			_End_I2C_Ram		; unused, only for ref of end of RAM allocation
                ENDC


#define	_SDA	_portb,0
#define	_SCL	_portb,1

#define	_SDA_TRIS	_trisb,0
#define	_SCL_TRIS	_trisb,1

;
#define	_START		SlvStatus,0
#define	_STOP		SlvStatus,1
#define	_ACK	      	SlvStatus,2

#define	_RW		SlvStatus,7
;
;**********************************************************************************************************
;				General Purpose Macros
;**********************************************************************************************************

RELEASE_BUS	MACRO
			bsf	_rp0		; select page 1
			bsf	_SDA		; tristate SDA
			bsf	_SCL		; tristate SCL
;			bcf	_Bus_Busy	; Bus Not Busy, TEMP ????, set/clear on Start & Stop
		ENDM

;**********************************************************************************************************
;			A MACRO To Load 8 OR 10 Bit Address To The Address Registers
;
;  SLAVE_ADDRESS is a constant and is loaded into the SlaveAddress Register(s) depending
;  on 8 or 10 bit addressing modes
;**********************************************************************************************************

LOAD_ADDR_10	MACRO	SLAVE_ADDRESS

		bsf	_10BitAddr	; Slave has 10 bit address        
		movlw	(SLAVE_ADDRESS & 0xff)
        	movwf	SlaveAddr		; load low byte of address  
		movlw	(((SLAVE_ADDRESS >> 7) & 0x06) | 0xF0)	   ; 10 bit addr is 11110XX0  
		movwf	SlaveAddr+1	; hi order  address

		ENDM

LOAD_ADDR_8	MACRO	SLAVE_ADDRESS

		bcf	_10BitAddr	; Set for 8 Bit Address Mode
		movlw	(SLAVE_ADDRESS & 0xff)
		movwf	SlaveAddr

                ENDM
